###################################################################################################
# _globals.py
#
# Implementation of Global Functions.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import Persistent
from Acquisition import Implicit
import string
import copy
import time

""" Globals. """

# Booleans.
# ---------
true = 1
false = 0


# -------------------------------------------------------------------------------------------------
# format_sort_id:
# -------------------------------------------------------------------------------------------------
def format_sort_id(i_sort_id):
  s_sort_id = "0000%i"%i_sort_id
  s_sort_id = "s" + s_sort_id[-4:]
  return s_sort_id


# -------------------------------------------------------------------------------------------------
# html_quote:
# -------------------------------------------------------------------------------------------------
def html_quote(v, name='(Unknown name)', md={}):
  from cgi import escape
  return escape(str(v), 1)


# -------------------------------------------------------------------------------------------------
# id_quote:
# -------------------------------------------------------------------------------------------------
def id_quote(srctext):
  dsttext = srctext
  for ch in range(0,255):
    if ch <= 32 or ch in [34,60,62] or ch >= 127:
      dsttext = string.replace(dsttext,chr(ch),'')
  return dsttext


# -------------------------------------------------------------------------------------------------
# js_quote:
# -------------------------------------------------------------------------------------------------
def js_quote(srctext):
  dsttext = srctext
  dsttext = string.replace(dsttext,"'","&quot;")
  dsttext = string.replace(dsttext,'"','&quot;')
  return dsttext


# -------------------------------------------------------------------------------------------------
# isPreviewRequest:
# -------------------------------------------------------------------------------------------------
def isPreviewRequest(REQUEST):
  return REQUEST is not None and REQUEST.has_key('preview') and REQUEST['preview'] not in ['',None] and not REQUEST.has_key('live')


# -------------------------------------------------------------------------------------------------
# isCacheRequest:
# -------------------------------------------------------------------------------------------------
def isCacheRequest(REQUEST):
  return not isPreviewRequest(REQUEST) and REQUEST.has_key('__cache__') and REQUEST['__cache__'] == 1


# -------------------------------------------------------------------------------------------------
# compareDate:
# -------------------------------------------------------------------------------------------------
def compareDate(t0, t1):
  YEAR = 0
  MONTH = 1
  DAY = 2
  if type(t0) is type(time.time()):
    t0 = time.localtime(t0)
  if type(t1) is type(time.time()):
    t1 = time.localtime(t1)
  y0 = t0[YEAR]
  m0 = t0[MONTH]
  d0 = t0[DAY]
  y1 = t1[YEAR]
  m1 = t1[MONTH]
  d1 = t1[DAY]
  if (y1 > y0) or \
     (y1 == y0 and m1 > m0) or \
     (y1 == y0 and m1 == m0 and d1 > d0):
    return +1
  elif (y1 == y0 and m1 == m0 and d1 == d0):
    return 0
  else:
    return -1


# -------------------------------------------------------------------------------------------------
# getDataSizeStr:
# -------------------------------------------------------------------------------------------------
def getDataSizeStr(size):
  i_mod = 0
  while size > 1024.0 and i_mod < 3:
    size = size / 1024.0
    i_mod = i_mod + 1
  s = str(float(int(size*10))/10)
  if i_mod == 0:
    s = "%s Bytes"%s
  elif i_mod == 1:
    s = "%s KB"%s
  elif i_mod == 2:
    s = "%s MB"%s
  elif i_mod == 3:
    s = "%s GB"%s
  return s


###################################################################################################
###################################################################################################
###
###   MyDebugger
###
###################################################################################################
###################################################################################################
class MyDebugger:

  def open(self):
    import tempfile
    self.s_tmpfile = tempfile.mktemp() + '.log'
    # self.s_value = ''
    print "[obenDbgFile]: s_tmpfile=%s"%self.s_tmpfile

  def write(self, s):
    print "[writeDbgFile]: %s"%s
    if hasattr(self,'s_tmpfile'):
      # self.s_value = self.s_value + s + '\n'
      self.s_value = s + '\n'
      fTmp = open(self.s_tmpfile,'a')
      fTmp.write(self.s_value)
      fTmp.close()

  def close(self):
    delattr(self,'s_tmpfile')
    # delattr(self,'s_value')

myDebugger = MyDebugger()


###################################################################################################
###################################################################################################
###
###   class MyClass:
###
###################################################################################################
###################################################################################################
class MyClass:
  pass


###################################################################################################
###################################################################################################
###
###   class MyDict:
###
###################################################################################################
###################################################################################################
class MyDict(Persistent,Implicit): 
    
    ###############################################################################################
    # MyDict.__init__: 
    #
    # Constructor (initialise a new instance of MyDict).
    ###############################################################################################
    def __init__(self): 
      return self

    ###############################################################################################
    # MyDict.addAttr: 
    ###############################################################################################
    def addAttr(self, id, label):
      setattr(self,id,label)

    ###############################################################################################
    # MyDict.delAttr: 
    ###############################################################################################
    def delAttr(self, id):
      if hasattr(self,id):
        delattr(self,id)

    ###############################################################################################
    # MyDict.getAttr: 
    #
    ###############################################################################################
    def getAttr(self, sId):
        return self.get()[sId]

    ###############################################################################################
    # MyDict.get: 
    #
    ###############################################################################################
    def get(self):
        return self.__dict__

    ###############################################################################################
    #  MyDict._getCopy: 
    #
    #  Clone object.
    ###############################################################################################
    def _getCopy(self):
         oClone = MyDict()
         for sId in self.__dict__.keys():
             attrvalue = self.__dict__[sId]
             try:
                 attrvalue = attrvalue._getCopy()
             except:
                 pass
             oClone.addAttr(sId,attrvalue)
         return oClone

    ###############################################################################################
    #  MyDict.dump: 
    #
    #  Dump object.
    ###############################################################################################
    def dump(self):
         #++ print "[dump]: %s"%str(self)
         for sId in self.__dict__.keys():
             #++ print "     %s=%s"%(sId,self.__dict__[sId])
             try:
                 self.__dict__[sId].dump()
             except:
                 pass


###################################################################################################
###################################################################################################
###
###   class MySectionizer
###
###################################################################################################
###################################################################################################
class MySectionizer:

    def __init__(self):
      self.sections = []

    def getLevel(self):
      return len(self.sections)

    def processLevel(self, level):
      # Increase section counter on this level.
      if level == len(self.sections):
        self.sections[level-1] = self.sections[level-1] + 1
      elif level > len(self.sections):
        for i in range(len(self.sections),level):
          self.sections.append(1)
      elif level < len(self.sections):
        for i in range(level,len(self.sections)):
          del self.sections[len(self.sections)-1]
        self.sections[level-1] = self.sections[level-1] + 1

    def __str__(self):
      s = ""
      for i in range(0,len(self.sections)):
        s = s + "%i."%self.sections[i]
      return s

    def clone(self):
      ob = MySectionizer()
      ob.sections = copy.deepcopy(self.sections)
      return ob


###################################################################################################
###################################################################################################
###
###   class MyStack
###
###################################################################################################
###################################################################################################
class MyStack:

    def __init__(self):
        self.clear()
        
    def toString(self):
        s = ''
        for el in self._stack:
            s = s + str(el) + ';'
        return s[:-1]

    def size(self):
        return len(self._stack)

    def clear(self):
        self._stack = []

    def push(self, x):
        self._stack.append(x)

    def pop(self):
        if len(self._stack) > 0:
            return self._stack.pop()
        else:
            return None

    def get(self, i=0):
        if len(self._stack) > 0 and abs(i) < len(self._stack):
            if i < 0:
               return self._stack[len(self._stack)+i-1]
            else:
               return self._stack[i]
        else:
            return None

    def top(self):
        if len(self._stack) > 0:
            return self._stack[len(self._stack)-1]
        else:
            return None

###################################################################################################
