###################################################################################################
# _metadata.py
#
# Implementation of class Metadata (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import string


###################################################################################################
###################################################################################################
###
###   C L A S S   M E T A D A T A
###
###################################################################################################
###################################################################################################
class Metadata:

    metadata_header = HTMLFile('dtml/metadict/header', globals()) 
    metadata_input_fields = HTMLFile('dtml/metadict/input_fields', globals()) 
    metadata_input_field = HTMLFile('dtml/metadict/input_field', globals()) 


    # ------------------------------------------------------------------------------------------
    #	Metadata.recurse_updateVersion:
    #
    #	Update version.
    # ------------------------------------------------------------------------------------------
    def metadatarecurse_updateVersion(self, lang=None, manage_lang=None, REQUEST=None):
      message = ''
      # Return with message.
      return message


    # ------------------------------------------------------------------------------------------
    #	Metadata.getMetaAttrs:
    # 
    #	Returns list of Object-Attributes of DC.Metadata.
    # ------------------------------------------------------------------------------------------
    def getMetaAttrs(self, REQUEST, meta_type=None):
    
      if meta_type is None:
        meta_type = self.meta_type
    
      attrs = []
      
      # Special Meta-Attributes.
      if 'attr_dc_coverage' in self.dGlobalAttrs[meta_type]['obj_attrs'].keys() and len(self.getLanguages()) > 1:
        attrs.append('attr_dc_coverage')
        
      # Meta-Attributes from Custom-Dictionaries.
      for attr in self.getMetadictAttrs(meta_type):
        metadictDetails = self.getMetadictDetails(attr)
        attrs.append(attr)
      
      return attrs


    # ---------------------------------------------------------------------------------------------
    #	Metadata.getDCValueList:
    #
    #	Values for select-list.
    # ---------------------------------------------------------------------------------------------
    def getDCValueList(self, attr, REQUEST):
    
        rtn = []
        
        ob = self.getDocumentElement()
          
        # Meta-Dictionaries.
        if attr in self.getMetadictAttrs(self.meta_type):
          metadictDetails = self.getMetadictDetails(attr)
          rtn.extend(metadictDetails.getKeys())
          
        # DC.Coverage.
        elif attr == 'dc_metadata_coverage_values':
          parent = self.getParentNode()
          if parent is None: parent = self
          parent_dc_coverage = parent.getObjProperty('attr_dc_coverage',REQUEST)
          if REQUEST['lang'] == self.getPrimaryLanguage():
            if parent_dc_coverage in ['','obligation']:
              rtn.extend(['obligation','proposal'])
            elif parent_dc_coverage in ['proposal']:
              rtn.extend(['proposal'])
          rtn.extend([REQUEST['lang']])
          
        elif not getattr(ob,attr,None) is None:
          for sLine in string.split(getattr(ob,attr),'\n'):
            sDCValue = string.strip(sLine)
            if len(sDCValue) > 0:
              rtn.append(sDCValue)
                    
        return rtn


    # ---------------------------------------------------------------------------------------------
    #	Metadata.acquireMetadata:
    #
    #	Acquires Metadata from parent.
    # ---------------------------------------------------------------------------------------------
    def acquireMetadata(self, key, lang, manage_lang, REQUEST):
      parent = self.getParentNode()
      if parent is not None and key in parent.getObjAttrs().keys():
        #++ print "[%s.acquireMetadata]: %s.%s=%s"%(self.meta_type,self.id,key,str(parent.getObjProperty(key,REQUEST)))
        self.setObjProperty(key,parent.getObjProperty(key,REQUEST),lang)


    # ---------------------------------------------------------------------------------------------
    #	Metadata.inheritMetadata:
    #
    #	Inherits Metadata recursive to children.
    # ---------------------------------------------------------------------------------------------
    def inheritMetadata(self, key, lang, manage_lang, REQUEST):
      for child in self.getChildNodes(REQUEST):
        if key in child.getObjAttrs().keys():
          #++ print "[%s.inheritMetadata]: %s.%s"%(child.meta_type,child.id,key)
          child.setObjProperty(key,self.getObjProperty(key,REQUEST),lang)
          child.inheritMetadata(key,lang,manage_lang,REQUEST)


    # ---------------------------------------------------------------------------------------------
    #	Metadata.setMetadata:
    #
    #	Changes Metadata.
    # ---------------------------------------------------------------------------------------------
    def setMetadata(self, lang, manage_lang, REQUEST):
    
    	#++ print "[%s.setMetadata]"%(self.meta_type)
    	if REQUEST.has_key('aq_dir'):
    	  print "[%s.setMetadata]: aq_dir=%s"%(self.meta_type,REQUEST['aq_dir'])
        
        # Acquire from top.
        if REQUEST.has_key('aq_dir') and REQUEST['aq_dir'] == 'top':
          if REQUEST.has_key('aq_ids'):
            for key in REQUEST['aq_ids']:
              self.acquireMetadata(key,lang,manage_lang,REQUEST)
                
        # Inherit to bottom.
        elif REQUEST.has_key('aq_dir') and REQUEST['aq_dir'] == 'bottom':
          if REQUEST.has_key('aq_ids'):
            for key in REQUEST['aq_ids']:
              self.inheritMetadata(key,lang,manage_lang,REQUEST)
                
        # Change.
        elif REQUEST.has_key('btn') and REQUEST['btn'] in [self.getLangStr('BTN_CHANGE',manage_lang),self.getLangStr('BTN_INSERT',manage_lang)]:
          for attr in self.getMetaAttrs(REQUEST):
            #++ print "[%s.setMetadata]: %s"%(self.meta_type,attr)
            if attr in self.getMetadictAttrs(self.meta_type):
              metadictDetails = self.getMetadictDetails(attr)
              key = metadictDetails.getAttr()
            else:
              key = attr
            self.setReqProperty(key,REQUEST)

###################################################################################################