###################################################################################################
# _objattrs.py
#
# Implementation of class ObjAttrs (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import copy
import string
import urllib
import time
# Product Imports.
import _globals
import _fileutil
import _metadictmanager
import _blobfields


# ---------------------------------------------------------------------------------------------
#  _objattrs.my_hasattr:
# ---------------------------------------------------------------------------------------------
def my_hasattr(ob, attr):
  try:
    return attr in ob.__dict__.keys()
  except:
    return hasattr(ob,attr)

# ---------------------------------------------------------------------------------------------
#  _objattrs.my_getattr:
# ---------------------------------------------------------------------------------------------
def my_getattr(ob, attr, default):
  if my_hasattr(ob,attr):
    return getattr(ob,attr)
  else:
    return default

# ---------------------------------------------------------------------------------------------
#  _objattrs.isLangAttr:
# ---------------------------------------------------------------------------------------------
def isLangAttr(obj_attr):
  datatype = obj_attr['datatype']
  return string.find(datatype,'Lang') == 0

# ---------------------------------------------------------------------------------------------
#  _objattrs.isInheritableLangAttr:
# ---------------------------------------------------------------------------------------------
def isInheritableLangAttr(obj_attr):
  return isLangAttr(obj_attr) and obj_attr['lang_inherit'] >= 0

# ---------------------------------------------------------------------------------------------
#  _objattrs.isEmptyAttr:
# ---------------------------------------------------------------------------------------------
def isEmptyAttr(value, datatype, obj_attr):
  return (value is None) or \
         (datatype in ['int'] and obj_attr.has_key('default') and obj_attr['default'] > 0 and value == 0) or \
  	 (datatype in ['string','html','text','url'] and len(str(value)) == 0)
      
# ---------------------------------------------------------------------------------------------
#  _objattrs.getObjDefault:
# ---------------------------------------------------------------------------------------------
def getObjDefault(datatype, obj_attr):
  default = None
  # Default by definition.
  if obj_attr.has_key('default'):
    default = obj_attr['default']
  # Default by datatype
  elif datatype in ['date','myimage','myfile']:
    default = None 
  elif datatype in ['dictionary']:
    default = {}
  elif datatype in ['list']:
    default = []
  elif datatype in ['int','boolean']:
    default = 0
  elif datatype in ['string','html','text','url']:
    default = ''
  # Return default.
  return default


###################################################################################################
###################################################################################################
###
###   O B J E C T   A T T R I B U T E S
###
###################################################################################################
###################################################################################################
class ObjAttrs:

    # ---------------------------------------------------------------------------------------------
    #	ObjAttrs.getObjAttrs:
    #
    #	Returns object-attributes and resolves meta-dictionaries.
    # ---------------------------------------------------------------------------------------------
    def getObjAttrs(self):
      
      ##### Meta-Objects / Special Documents ####
      if self.meta_type == 'ZMSCustom':
        obj_attrs = self.__obj_attrs__.copy()
        for attr in self.getMetaobj(self.meta_id)['attrs']:
          if attr['type'][:3] != 'ZMS':
            dct = {}
            dct['datatype'] = attr['type']
            obj_attrs[attr['id']] = dct
      
      ##### Object-Attributes (incl. Meta-Attributes) ####
      else:
        doc = self.getDocumentElement()
        metadict_attrs = getattr(doc,'__metadict_attrs__',None)
        if metadict_attrs is None:
          doc.synchronizeObjAttrs()
          metadict_attrs = getattr(doc,'__metadict_attrs__',None)
        if metadict_attrs is not None:
          obj_attrs = metadict_attrs[metadict_attrs.index(self.meta_type)+1]
        else:
          obj_attrs = self.__obj_attrs__
      
      # Return object-attributes.
      return obj_attrs

    # ---------------------------------------------------------------------------------------------
    #	ObjAttrs.findObjAttr:
    # ---------------------------------------------------------------------------------------------
    def findObjAttr(self, key):
      
      ##### Meta-Objects / Special Documents ####
      if self.meta_type == 'ZMSCustom':
        obj_attrs = self.__obj_attrs__
        if obj_attrs.has_key(key):
          return obj_attrs[key].copy()
        obj_attrs = self.__obj_attrs__
        for attr in self.getMetaobj(self.meta_id)['attrs']:
          if attr['id'] == key:
            return {'datatype':attr['type']}
      
      ##### Object-Attributes (incl. Meta-Attributes) ####
      else:
        obj_attrs = self.getObjAttrs()
        if obj_attrs.has_key(key):
          return obj_attrs[key].copy()
      
      # Return empty Object-Attribute.
      return {}

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttr:
    # ---------------------------------------------------------------------------------------------
    def getObjAttr(self, key):
      
      obj_attr = self.findObjAttr(key)
      
      # Defaults.
      
      #-- KEY
      obj_attr['key'] = key
      #-- NATIVE
      if not obj_attr.has_key('native'): 
        obj_attr['native'] = 0
      #-- DATATYPE
      if not obj_attr.has_key('datatype'): 
        obj_attr['datatype'] = 'string'
      #-- TYPE
      if not obj_attr.has_key('type'): 
        if obj_attr['datatype'] == 'boolean':
          obj_attr['type'] = 'checkbox'
        else:
          obj_attr['type'] = 'input'
      #-- SIZE
      if not obj_attr.has_key('size'): 
        if self.getObjDatatype(obj_attr) == 'int':
          obj_attr['size'] = 4
        else:
          obj_attr['size'] = 20
      #-- DEFAULT
      if not obj_attr.has_key('default'): 
        if obj_attr['datatype'] == 'boolean':
          obj_attr['default'] = 1
      #-- LANG_INHERIT
      if not obj_attr.has_key('lang_inherit'): 
        obj_attr['lang_inherit'] = 1
      
      # Return Object-Attribute.
      return obj_attr


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjOptions:
    # ---------------------------------------------------------------------------------------------
    def getObjOptions(self, key, REQUEST):
      
      options = []
      options_tup = []
      
      obj_attr = self.getObjAttr(key)
      if obj_attr.has_key('options'):
        obj_attrOptions = obj_attr['options']
        if type(obj_attrOptions) is type([]):
          for i in range(0,len(obj_attrOptions)/2):
            options.append((obj_attrOptions[i*2],obj_attrOptions[i*2+1]))
        elif type(obj_attrOptions) is type({}):
          for k in obj_attrOptions.keys():
            options.append((k,obj_attrOptions[k]))
        
        for option in options:
          lang_attr = 'OPT_'
          for skey in string.split(key,'_'):
            lang_attr = lang_attr + skey[0]
          lang_attr = lang_attr + '_' + string.replace(str(option[1]),' ','')
          lang_attr = string.upper(lang_attr)
          options_tup.append((str(option[0]),self.getLangStr(lang_attr,REQUEST['manage_lang'])))
      
      return options_tup

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjKey:
    # ---------------------------------------------------------------------------------------------
    def getObjKey(self, obj_attr):
      return obj_attr['key']

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjDatatype:
    # ---------------------------------------------------------------------------------------------
    def getObjDatatype(self, obj_attr):
      datatype = string.lower(obj_attr['datatype'])
      if isLangAttr(obj_attr):
        datatype = datatype[4:]
      return datatype

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjAttrName:
    # ---------------------------------------------------------------------------------------------
    def getObjAttrName(self, obj_attr, lang=None):
      attr = self.getObjKey(obj_attr)
      if isLangAttr(obj_attr):
        attr = '%s_%s'%(attr,lang)
      return attr

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.isEnabledAttr:
    # ---------------------------------------------------------------------------------------------
    def isEnabledAttr(self, key, REQUEST):
      obj_attr = self.getObjAttr(key)
      return (self.getAutocommit()) or \
             (REQUEST.has_key('insert') and REQUEST['insert'] == 1) or \
             (REQUEST.has_key('lang') and REQUEST['lang'] == self.getPrimaryLanguage()) or \
             (self.getDCCoverage(REQUEST) != 'obligation') or \
             (isLangAttr(obj_attr) and obj_attr['lang_inherit'])


    ###############################################################################################
    #
    #  I N T E R F A C E
    #
    ###############################################################################################

    # ---------------------------------------------------------------------------------------------
    #  getObjInput:
    # ---------------------------------------------------------------------------------------------
    def getObjInput(self, key, REQUEST):
      
      obj_attr = self.getObjAttr(key)
      #-- TYPE(S)
      type = obj_attr['type']
      datatype = self.getObjDatatype(obj_attr)
      #-- NAME
      fmName = 'form0'
      elName = self.getObjAttrName(obj_attr,REQUEST['lang'])
      #-- VALUE
      value = self.getObjProperty(key,REQUEST)
      #-- ENABLED
      enabled = self.isEnabledAttr(key,REQUEST)
      
      #-- INPUT
      if type in ['input']: 
        if datatype == 'date':
          return self.getDateInput(fmName,elName,value,enabled,REQUEST)
        else:
          return self.getTextInput(fmName,elName,obj_attr['size'],value)
      
      #-- TEXTAREA
      elif type in ['textarea']:
        return self.getTextArea(fmName,elName,50,15,value,enabled,REQUEST)
      
      #-- CHECKBOX
      elif type in ['checkbox']:
        return self.getCheckbox(fmName,elName,value,1,enabled,REQUEST)
      
      #-- SELECT
      elif type in ['select','lookup','multiselect','multilookup']:
        options = self.getObjOptions(key,REQUEST)
        multiple = string.find(type,'multi') == 0
        return self.getSelect(fmName,elName,value,options,multiple,enabled,REQUEST)
      
      # Return with error-message.
      return '<font class="form-text" style="color:red">%s</font>'%key
    
    
    """
    ###############################################################################################
    #
    #  G E T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.blob_html:
    # ---------------------------------------------------------------------------------------------
    def blob_html(self, key, REQUEST, RESPONSE):
      """ ObjAttrs.blob_html """
      blob = self.getObjProperty(key,REQUEST)
      html = blob.index_html(REQUEST,RESPONSE)
      # increase counter for downloads
      if 'counter' in self.getObjAttrs().keys():
        for preview in [None,'preview']:
          ob = self.getObjVersion({'preview':preview})
          attr = 'counter_%s'%REQUEST['lang']
          setattr(ob,attr,getattr(ob,attr,0)+1)
      # return export for download to browser
      RESPONSE.setHeader('Content-Type',getattr(blob,'content_type','Unknown'))
      RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%getattr(blob,'filename',''))
      return html

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.hasObjProperty:
    #
    #  Checks if object has specified property.
    # ---------------------------------------------------------------------------------------------
    def hasObjProperty(self, key, REQUEST):
      #-- DEFINITION
      obj_attr = self.getObjAttr(key)
      #-- OBJECT (Live/Preview).
      ob = self.getObjVersion(REQUEST)
      #-- ATTR
      attr = self.getObjAttrName(obj_attr,REQUEST['lang'])
      #-- Return if object has property (0=false, 1=true).
      return my_hasattr(ob,attr)

    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjProperty:
    #
    #  Retrieves value for specified property.
    # ---------------------------------------------------------------------------------------------
    def getObjProperty(self, key, REQUEST=None, default=None):

      #-- DEFINITION    
      obj_attr = self.getObjAttr(key)

      #-- DATATYPE
      datatype = self.getObjDatatype(obj_attr)

      #-- VALUE
      value = self.getObjValue(obj_attr,datatype,REQUEST)

      #-- DEFAULT        
      
      # Default to primary-language.
      if (REQUEST is not None and REQUEST.has_key('lang') and REQUEST['lang'] != self.getPrimaryLanguage()) and \
         (isLangAttr(obj_attr)) and \
         (isInheritableLangAttr(obj_attr)) and \
         (isEmptyAttr(value,datatype,obj_attr) or not self.isEnabledAttr(key,REQUEST)):
        req = {}
        req['lang'] = self.getPrimaryLanguage()
        if _globals.isPreviewRequest(REQUEST) and not _globals.isCacheRequest(REQUEST):
          req['preview'] = REQUEST['preview']
        value = self.getObjProperty(key,req)
      
      # Default by datatype.
      if isEmptyAttr(value,datatype,obj_attr):
        value = getObjDefault(datatype,obj_attr)

      # Correct by datatype.
      if datatype in ['int','string','html','text','url']:
        if type(value) is type([]):
          if len(value) == 0:
            value = getObjDefault(datatype,obj_attr)
          else:
            value = value[0]

      if datatype in ['date']:
        if value is not None:
          lang = REQUEST['lang']
          if type(value) is type(time.time()): 
            value = time.localtime(value)
            self.setObjProperty(key,value,lang)
          elif type(value) is type(''): 
            value = self.parseLangFmtDate(value,lang,'DATE_FMT')
            self.setObjProperty(key,value,lang)

      # Return value.
      #++ _globals.myDebugger.write("[%s.getObjProperty]: %s[%s]=%s[%s]"%(self.meta_type,key,datatype,str(value),str(default)))
      return value


    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.getObjValue:
    #
    #  Retrieves value for specified attribute.
    # ---------------------------------------------------------------------------------------------
    def getObjValue(self, obj_attr, datatype, REQUEST):
    
        #-- OBJECT (Live/Preview).
        ob = self.getObjVersion(REQUEST)
        
        #-- ATTR
        lang = self.getPrimaryLanguage()
        if REQUEST is not None and REQUEST.has_key('lang'):
          lang = REQUEST['lang']
        attr = self.getObjAttrName(obj_attr,lang)
        
        #-- VALUE
        value = None
        if my_hasattr(ob,attr):
          value = my_getattr(ob,attr,value)
        
        #-- Blob-Fields
        if datatype in ['myimage','myfile']:
          if value is not None and type(value) is type(''):
            value = None
            
        #-- Integer-Fields.
        elif datatype in ['int','boolean']:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = int(value)

        # Return value.
        #++ _globals.myDebugger.write("[%s.getObjValue]: %s[%s]=%s"%(self.meta_type,attr,datatype,str(value)))
        return value


    """
    ###############################################################################################
    #
    #  S E T
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  setReqProperty:
    #
    #  Assigns value to specified property from Request-Object.
    # ---------------------------------------------------------------------------------------------
    def setReqProperty(self, key, REQUEST):
        lang = REQUEST['lang']
        manage_lang = REQUEST['manage_lang']
        obj_attr = self.getObjAttr(key)
        
        #-- DATATYPE
        datatype = self.getObjDatatype(obj_attr)
        
        #-- ATTR
        attr = self.getObjAttrName(obj_attr,lang)
        
        #-- VALUE
        set, value = 0, None
        if REQUEST.has_key(attr):
          value = REQUEST[attr]
        #++ _globals.myDebugger.write("[%s.setReqProperty]: %s(%s)=%s"%(self.meta_type,str(attr),str(datatype),str(value)))
        
        #-- Date-Fields
        if datatype in ['date']:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = self.parseLangFmtDate(value,manage_lang,'DATE_FMT')
          else:
            value = None
          set = 1
            
        #-- Data-Fields (Image/File)
        elif datatype in ['myimage','myfile']:
        
          # Dimensions
          if datatype in ['myimage'] and self.hasObjProperty(key,REQUEST):
            img = self.getObjProperty(key,REQUEST)
            if img is not None:
              if REQUEST.has_key('width_%s'%attr): 
                img.width = REQUEST['width_%s'%attr]
              if REQUEST.has_key('height_%s'%attr): 
                img.height = REQUEST['height_%s'%attr]
                
          # Delete
          if REQUEST.has_key('del_%s'%attr) and int(REQUEST['del_%s'%attr]) == 1:
            set, value = 1, None
            #++ _globals.myDebugger.write("[%s.setReqProperty]: %s(%s) del_attr"%(self.meta_type,str(attr),str(datatype)))
            
          # Upload
          elif value is not None and len(getattr(value,'filename','')) > 0:
            set, value = 1, value
            #++ _globals.myDebugger.write("[%s.setReqProperty]: %s(%s) set_attr=%s"%(self.meta_type,str(attr),str(datatype),str(getattr(value,'filename',''))))
            
          # Reset
          elif REQUEST.has_key('insert'):
            set, value = 1, None
        
        #-- Dictionary-Fields
        elif datatype in ['dictionary']:
          if value is not None:
            if type(value) is type({}):
              set, value = 1, value
        
        #-- List-Fields
        elif datatype in ['list']:
          if value is not None:
            if type(value) is type(''):
              value = [string.strip(value)]
            if type(value) is type([]):
              set, value = 1, value
        
        #-- String-Fields
        elif datatype in ['string','html','text','url']:
          if value is not None:
            set, value = 1, value
        
        #-- Integer-Fields
        elif datatype in ['int','boolean']:
          if value is not None:
            if type(value) == type('') and len(value) == 0:
              set, value = 1, None
            else:
              set, value = 1, int(value)
          else:
            set, value = 1, 0
        
        #-- SET?
        if set == 1: self.setObjProperty(key,value,lang)        

    
    # ---------------------------------------------------------------------------------------------
    #  setObjProperty:
    #
    #  Assigns value to specified property.
    # ---------------------------------------------------------------------------------------------
    def setObjProperty(self, key, value, lang=None):
	obj_attr = self.getObjAttr(key)
	
        #-- OBJECT (Apply changes always to Work-Version!).
        req = {'preview':'preview'}
        ob = self.getObjVersion(req)

        #-- DATATYPE [RAW]
        datatype = self.getObjDatatype(obj_attr)
        #-- ATTR
        attr = self.getObjAttrName(obj_attr,lang)

        #-- VALUE
        if type(value) is type(''):
          value = string.strip(value)
          while len(value) > 0 and value[-1] == '\n':
            value = value[:-1]
        # Retrieve value from options.
        if obj_attr.has_key('options'):
          options = obj_attr['options']
          try: 
            i = options.index(int(value))
            if i%2==1: value = options[i-1]
          except:
            try:
              i = options.index(str(value))
              if i%2==1: value = options[i-1]
            except: 
              pass
        
        #-- Data-Fields (File/Image)
        if datatype in ['myimage','myfile']:
          bk_value = value
          if not bk_value is None and len(str(bk_value)) > 0:
            value = _blobfields.createBlobField(datatype)
            if not value is None:
              value.manage_upload(bk_value)
              value.filename = _fileutil.extractFileName(getattr(bk_value,'filename',None))
            
        #-- Date-Fields
        elif datatype in ['date']:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = self.parseLangFmtDate(value,lang,'DATE_FMT')
            
        #-- Integer-Fields
        elif datatype in ['int','boolean']:
          if value is not None and type(value) is type('') and len(value) > 0:
            value = int(value)
            
        #-- String-Fields.
        elif datatype in ['string','html','text','url']:
          if value is None:
            value = ''
          else:
            if type(value) is type(''):
              value = string.strip(value)

        #-- SET
        if datatype in ['myimage','myfile']:
          #++ _globals.myDebugger.write("[%s.setObjProperty]: %s=<%s>"%(ob,str(attr),str(datatype)))
          pass
        else:
          #++ _globals.myDebugger.write("[%s.setObjProperty]: %s=%s"%(ob,str(attr),str(value)))
          pass
          
        setattr(ob,attr,value)


    """
    ###############################################################################################
    #
    #  V E R S I O N
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  __hasObjProperty__:
    #
    #  Only for internal use: checks for specified property.
    # ---------------------------------------------------------------------------------------------
    def __hasObjProperty__(self, attr):
      return hasattr(self,attr) or hasattr(self.getObjVersion({'preview':'preview'}),attr)

    
    # ---------------------------------------------------------------------------------------------
    #  __delObjProperty__:
    #
    #  Only for internal use: delete specified property.
    # ---------------------------------------------------------------------------------------------
    def __delObjProperty__(self, attr):
      try: delattr(self,attr)
      except: pass
      try: delattr(self.getObjVersion({'preview':'preview'}),attr)
      except: pass

    
    # ---------------------------------------------------------------------------------------------
    #  __initObjProperty__:
    #
    #  Only for internal use: init specified property with value.
    # ---------------------------------------------------------------------------------------------
    def __initObjProperty__(self, key, value, lang=None):
      #-- ATTR
      obj_attr = self.getObjAttr(key)
      attr = self.getObjAttrName(obj_attr,lang)
      #-- INIT
      setattr(self,attr,value)
      setattr(self.getObjVersion({'preview':'preview'}),attr,value)
    

    """
    ###############################################################################################
    #
    #  C L O N E
    #
    ###############################################################################################
    """
    
    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs._cloneObjAttr:
    #
    #  Clone concrete single object-attribute.
    # ---------------------------------------------------------------------------------------------
    def _cloneObjAttr(self, src, dst, key, lang=None):
      obj_attr = self.getObjAttr(key)
      #-- DATATYPE
      datatype = self.getObjDatatype(obj_attr)
      #-- ATTR
      attr = self.getObjAttrName(obj_attr,lang)
      #-- DEFAULT
      default = getObjDefault(datatype,obj_attr)
      #-- VALUE
      value = my_getattr(src,attr,default)
      #-- CLONE
      if not value is None:
        if datatype in ['myimage','myfile']:
          value = value._getCopy()
        else:
          try:
            value = copy.deepcopy(value)
          except:
            try:
              value = value.copy()
            except:
              print "[%s._cloneObjAttr]: WARNING - Exception in %s(%s): value=%s"%(self.meta_type,attr,datatype,str(value))
              value = default
      # Assign value.
      setattr(dst,attr,value)
      
    # ---------------------------------------------------------------------------------------------
    #  ObjAttrs.cloneObjAttrs:
    #
    #  Clone object-attributes.
    # ---------------------------------------------------------------------------------------------
    def cloneObjAttrs(self, src, dst, REQUEST=None):
      obj_attrs = self.getObjAttrs()
      
      lang = None
      if REQUEST is not None and REQUEST.has_key('lang'):
        lang = REQUEST['lang']
      
      for key in obj_attrs.keys():
        
        #-- Multi-Language Attributes.
        obj_attr = self.getObjAttr(key)
        if isLangAttr(obj_attr):
          for s_lang in self.getLanguages():
            if lang == None or lang == s_lang:
              self._cloneObjAttr(src,dst,key,s_lang)
        
        #-- Others.
        else:
          if lang == None or lang == self.getPrimaryLanguage():
            self._cloneObjAttr(src,dst,key)


    """
    ###############################################################################################
    #
    #  D a t a   ( I m a g e / F i l e )
    #
    ###############################################################################################
    """

    #----------------------------------------------------------------------------------------------
    #	ObjAttrs.recurse_downloadRessources:
    #
    #	Download from ZODB to file-system during Export.
    #----------------------------------------------------------------------------------------------
    def recurse_downloadRessources(self, folder='.'):
      obj_attrs = self.getObjAttrs()
      #++ _globals.myDebugger.write("[%s.recurse_downloadRessources]: %s"%(self.meta_type,self.id))
      message = ''
      # Attributes.
      for key in obj_attrs.keys():
        obj_attr = self.getObjAttr(key)
        datatype = self.getObjDatatype(obj_attr)
        if datatype in ['myimage','myfile']:
          for s_lang in self.getLanguages():
            try:
              req = {}
              req['lang'] = s_lang
              req['preview'] = 'preview'
              ob = self.getObjValue(obj_attr,datatype,req)
              if ob is not None: ob.download(self,folder)
            except:
              _globals.myDebugger.write("#####[%s.recurse_downloadRessources]: Exception in %s.%s_%s!#####"%(self.meta_type,self.id,key,s_lang))
              pass
      # Process children.
      if len(self.objectValues()) > 0:
        for child in self.getChildNodes():
          message = message + child.recurse_downloadRessources(folder)
      # Return message.
      return message


    #----------------------------------------------------------------------------------------------
    #	ObjAttrs.recurse_uploadRessources:
    #
    #	Upload from file-system to ZODB during Import.
    #----------------------------------------------------------------------------------------------
    def recurse_uploadRessources(self, folder='.'):
      obj_attrs = self.getObjAttrs()
      #++ _globals.myDebugger.write("[%s.recurse_uploadRessources]: %s"%(self.meta_type,self.id))
      message = ''
      # Attributes.
      for key in obj_attrs.keys():
        obj_attr = self.getObjAttr(key)
        datatype = self.getObjDatatype(obj_attr)
        if datatype in ['myimage','myfile']:
          for s_lang in self.getLanguages():
            try:
              req = {}
              req['lang'] = s_lang
              req['preview'] = 'preview'
              ob = self.getObjValue(obj_attr,datatype,req)
              if ob is not None: ob.upload(self,folder)
            except:
              _globals.myDebugger.write("*****[%s.recurse_uploadRessources]: Exception in %s.%s_%s!*****"%(self.meta_type,self.id,key,s_lang))
              pass
      # Process children.
      if len(self.objectValues()) > 0:
        for child in self.getChildNodes():
          message = message + child.recurse_uploadRessources(folder)
      # Return message.
      return message

###################################################################################################