###################################################################################################
# _printablemanager.py
#
# Implementation of class PrintableManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import string
import urllib
import ftplib
import tempfile
import stat
import os
from OFS.Image import Image
# Product Imports.
from _globals import *
import _exportable
import _fileutil
import _xmllib


###################################################################################################
# exportPrintableHtml:
###################################################################################################
def exportPrintableHtml(self, sectionize, REQUEST):
  print "[exportPrintableHtml]"
        
  level = 0
      
  # Build <html>.
  html = ""
  html = html + '<html>'
  html = html + '<head>'
  html = html + '<title>%s</title>\n'%(self.getTitle(REQUEST))
  html = html + '<link rel="schema.DC" href="http://purl.org/DC/elements/1.1/" title="http://purl.org/DC/elements/1.1/">\n'
  html = html + '<link rel="stylesheet" href="%s/f_css_printhtml">\n'%self.getDocumentElement().absolute_url()
  html = html + '</head>\n'
  html = html + '<body>\n'
  html = html + self.printHtml(level,sectionize,MySectionizer(),REQUEST)
  html = html + '</body>\n'
  html = html + '</html>\n'
  
  return html


###################################################################################################
###################################################################################################
###
###   class PrintableManager
###
###################################################################################################
###################################################################################################
class PrintableManager: 
        
    # ---------------------------------------------------------------------------------------------
    #	PrintableManager.toPrintableRtf:
    # ---------------------------------------------------------------------------------------------
    def toPrintableRtf(self, REQUEST):
      print "[%s.toPrintableRtf]: "%(self.meta_type)
        
      # Create temporary folder.
      tmpfolder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=0,REQUEST=REQUEST)

      # Export printable <html> to temporary file.
      html = exportPrintableHtml(self,0,REQUEST)
      html = _exportable.localHtml(self,self,html,REQUEST)
      html = _exportable.localPrintHtml(self,self,html,REQUEST)

      # Write html to file.
      tmpfilename = _fileutil.getOSPath('%s/printable.html'%tmpfolder)
      _fileutil.exportObj(html,tmpfilename)
  

      # Generate RTF file.
      # ------------------
      filename = _fileutil.getOSPath('%s/printable'%tmpfolder)
      ##### assemble command line
      command = "java -cp " \
              + _fileutil.getOSPath("%s/Products/zms/bin/java/lib/zms.jar"%(SOFTWARE_HOME)) \
              + " xml2rtf.Batch" \
              + " %s.html %s.rtf > %s.log"%(filename,filename,filename)
      ##### verbose command-line
      print command
      ##### execute command-line
      os.system(command)

      # Get zip-file.
      zipfiles = _fileutil.getOSPath('%s.rtf'%filename)
      rtn = _fileutil.buildZipArchive(zipfiles)

      # Remove temporary folder.
      _fileutil.rmFilePath(tmpfolder)
        
      return rtn
    
    # ---------------------------------------------------------------------------------------------
    #	PrintableManager.toPrintablePdf:
    # ---------------------------------------------------------------------------------------------
    def toPrintablePdf(self, REQUEST):
      print "[%s.toPrintablePdf]: "%(self.meta_type)
        
      # Create temporary folder.
      tmpfolder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=0,REQUEST=REQUEST)

      # Export printable <html> to temporary file.
      html = exportPrintableHtml(self,1,REQUEST)
      html = _exportable.localHtml(self,self,html,REQUEST)
      html = _exportable.localPrintHtml(self,self,html,REQUEST)

      # Write html to file.
      tmpfilename = _fileutil.getOSPath('%s/printable.html'%tmpfolder)
      _fileutil.exportObj(html,tmpfilename)
  
      # Generate PDF file.
      # ------------------
      # HTMLDOC
      #   --size <A4>                 	Page size DIN A4.
      #   --book                     	Specifies that the input files comprise a book with chapters and heading.
      #   --toctitle <string>		Specifies the String to display at the top of the 
      #					table-of-contents.
      #   --logoimage <filename>	Specifies the Logo image for the HTML navigation bar and
      #					page headers and footers (supported formats are GIF, JPEG
      #					and PNG).
      #   --header lcr			Specifies the contents of the page header.
      #					The lcr parameter is a three-character string representing
      #					the left, center and right header fields.
      #					Each character can be one of the following:
      #					.	Blank
      #					1	Current page number in decimals (1, 2, 3,...)
      #					c	Current chapter title
      #					C	Current chapter page number
      #					h	Current heading
      #					i	Current page number in lowercase romans (i, ii, iii,...)
      #					I	Current page number in uppercase romans (i, ii, iii,...)
      #					l	Logo image
      #					t	Document title
      #   -t <pdf13>            	Specifies the output format for the document.
      #   -f <filename>         	Specifies an output file for the document.
   
      # >>> Important Note:
      # >>> DOS, Win32: <HTML>DOC must be installed in the path.
      # >>> UNIX, Linux, Solaris: ...
      command = "htmldoc"
           
      ##### assemble command line
      filename = '%s/printable'%tmpfolder
      command = command \
              + ' --size A4' \
              + ' --book' \
              + ' --toctitle "%s"'%(self.getLangStr('ATTR_TOC',REQUEST['lang'])) \
              + ' --header t.l' \
              + ' -t pdf13' \
              + ' -f %s.pdf'%(filename) \
              + ' %s.html'%(filename)
      ##### verbose command-line
      print command
      ##### execute command-line
      os.system(command)

      # Get zip-file.
      zipfiles = _fileutil.getOSPath('%s.pdf'%filename)
      rtn = _fileutil.buildZipArchive(zipfiles)

      # Remove temporary folder.
      _fileutil.rmFilePath(tmpfolder)
        
      return rtn

    
    # ---------------------------------------------------------------------------------------------
    #	PrintableManager.toZippedPrintableHtml:
    # ---------------------------------------------------------------------------------------------
    def toZippedPrintableHtml(self, REQUEST):
      print "[%s.toZippedPrintableHtml]: "%(self.meta_type)

      # Create temporary folder.
      tmpfolder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=0,REQUEST=REQUEST)

      # Export printable <html> to temporary file.
      html = exportPrintableHtml(self,0,REQUEST)
      html = _exportable.localHtml(self,self,html,REQUEST)
      html = _exportable.localPrintHtml(self,self,html,REQUEST)
      
      # Write css to file.
      f_css_printhtml_filename = _fileutil.getOSPath('%s/f_css_printhtml'%tmpfolder)
      f = open(f_css_printhtml_filename,'w')
      f.write(self.f_css_printhtml())
      f.close()

      # Write html to file.
      tmpfilename = _fileutil.getOSPath('%s/printable.html'%tmpfolder)
      _fileutil.exportObj(html,tmpfilename)

      # Get zip-file.
      zipfiles = _fileutil.getOSPath('%s/*.*'%tmpfolder)
      rtn = _fileutil.buildZipArchive(zipfiles)

      # Remove temporary folder.
      _fileutil.rmFilePath(tmpfolder)
        
      return rtn


    # ---------------------------------------------------------------------------------------------
    #	PrintableManager.toPrintableHtml:
    # ---------------------------------------------------------------------------------------------
    def toPrintableHtml(self, REQUEST):
      html = exportPrintableHtml(self,1,REQUEST)
      html = _exportable.localPrintHtml(self,self,html,REQUEST)
      return html

###################################################################################################
