###################################################################################################
# zmscustom.py
#
# Implementation of class ZMSCustom (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import string 
import urllib
import time
# Product Imports.
from zmscontainerobject import ZMSContainerObject
from zmsobject import ZMSObject
import _globals


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

def manage_addZMSCustom(self, custom, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
  """ manage_addZMSCustom """
  
  # Meta-ID.
  meta_id = None
  meta_name = custom
  for id in self.getMetaobjIds(REQUEST):
    ob = self.getMetaobj(id)
    if meta_id is None and \
       meta_name == ob['name']:
      meta_id = id
  
  ##### Create ####
  obj = ZMSCustom(self.getNewId(),_sort_id+1,meta_id)
  self._setObject(obj.id, obj)
  
  for oItem in self.objectValues():
    if oItem.id == obj.id:
      ##### Object State ####
      oItem.setObjStateNew(REQUEST)
      ##### Init Properties ####
      for s_lang in self.getLanguages():
        oItem.setObjProperty('active',_globals.true,s_lang)
      oItem.manage_changeProperties(lang,manage_lang,REQUEST,RESPONSE)
      ##### VersionManager ####
      oItem.__autoCommit__(REQUEST)
  
  ##### Normalize Sort-IDs ####
  self.normalizeSortIds()
  
  # Return with message.        
  message = self.getLangStr('MSG_INSERTED',manage_lang)%meta_name
  RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################

class ZMSCustom(ZMSContainerObject): 
        
    # Properties.
    # -----------
    meta_type = "ZMSCustom"
    icon = "misc_/zms/zmsdocument.gif"
    icon_disabled = "misc_/zms/zmsdocument_disabled.gif"

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',    'action': 'manage_main'},
	{'label': 'TAB_ACCESS',  'action': 'manage_users'},
	{'label': 'TAB_PREVIEW', 'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_properties','manage_changeProperties',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        'sort_id':{'datatype':'string','native':1},
        'meta_id':{'datatype':'string','native':1},
        'change_uid':{'datatype':'LangString','lang_inherit':-1},
        'change_dt':{'datatype':'LangDate','lang_inherit':-1},
        'work_uid':{'datatype':'LangString','lang_inherit':-1},
        'work_dt':{'datatype':'LangDate','lang_inherit':-1},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':0},
        'attr_active_start':{'datatype':'LangDate','lang_inherit':0},
        'attr_active_end':{'datatype':'LangDate','lang_inherit':0},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/metaobj/manage_main', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSCustom.__init__: 
    #
    # Constructor (initialise a new instance of ZMSCustom).
    ###############################################################################################
    def __init__(self, id='', sort_id='', meta_id=''): 
      """ ZMSCustom.__init__ """
      ZMSObject.__init__(self,id,sort_id)
      self.meta_id = meta_id


    ###############################################################################################
    # ZMSCustom.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
      message = ''

      # Process super-objects.
      message = message + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        
      # Process object.
      # [...]

      # ---------------------------------------------------------------------
      # 13.02.2002
      # Build #053: Activation period.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '053':
        for s_lang in self.getLanguages():
  	  if self.__hasObjProperty__('active_%s'%s_lang):
	    if getattr(self,'active_%s'%s_lang,0)==1 and \
	       not hasattr(self,'active_start_dat_%s'%s_lang):
              #++ print "[%s.recurse_updateVersion]: Active (Start-Date) =%i"%(self.meta_type,time.time())
     	      self.__initObjProperty__('attr_active_start',time.time(),s_lang)
	    self.__delObjProperty__('active_%s'%s_lang)
	  
      # ---------------------------------------------------------------------
      # 20.02.2002
      # Build #061: Make object-attributes multilingual.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '061':
        attrs = []
        for attr in self.getMetaobj(self.meta_id)['attrs']:
	  if attr['type'][:3] != 'ZMS':
            attrs.append(attr['id'])
        for key in attrs:
          if key in self.getObjAttrs().keys():
            for s_preview in [None,'preview']:
              req = {'preview':s_preview}
              ob = self.getObjVersion(req)
              if hasattr(ob,key):
                for s_lang in self.getLanguages():
                  attr = '%s_%s'%(key,s_lang)
                  value = getattr(ob,key)
                  #++ print "[%s.recurse_updateVersion]: %s.%s=%s"%(self.meta_type,str(ob),attr,str(value))
                  setattr(ob,attr,value)
	    self.__delObjProperty__(key)

      # ---------------------------------------------------------------------
      # 26.02.2002
      # Build #065: Correct bug in #064.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '064':
        ob_live = self.getObjVersion({'preview':None})
        ob_work = self.getObjVersion({'preview':'preview'})
        for key in attrs:
          for s_lang in self.getLanguages():
            attr = '%s_%s'%(key,s_lang)
            if hasattr(ob_live,attr):
              value = getattr(ob_live,attr)
              if not hasattr(ob_work,attr) or \
                 getattr(ob_work,attr) != value and len(self.getObjStates()) == 0:
                #++ print "[%s.recurse_updateVersion]: Rescue %s..."%(self.meta_type,attr)
                setattr(ob_work,attr,value)

      # ---------------------------------------------------------------------
      # 26.02.2002
      # Build #067: Activity-Toggle.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '067':
        for s_preview in [None,'preview']:
          req = {'preview':s_preview}
          ob = self.getObjVersion(req)
          for s_key in ['active']:
            for s_lang in self.getLanguages():
              attr = '%s_%s'%(s_key,s_lang)
              if not hasattr(ob,attr) or getattr(self,'build','') <= '067':
                active = getattr(ob,'attr_active_start_%s'%s_lang,None) is not None or \
                         getattr(ob,'attr_active_end_%s'%s_lang,None) is not None 
  	        #++ print "[%s.recurse_updateVersion]: Activity-Toggle %s.%s=%i"%(self.meta_type,str(ob),attr,active)                       
                setattr(ob,attr,active)

      # Recurse through child-nodes.
      for ob in self.getChildNodes():
        message = message + ob.recurse_updateVersion(lang,manage_lang,REQUEST)
      
      # Return with message.
      return message

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.initChildNodes
    # ---------------------------------------------------------------------------------------------
    def initChildNodes(self, REQUEST):
      for obj_attr in self.getMetaobj(self.meta_id)['attrs']:
        type = obj_attr['type']
        if type in self.dGlobalAttrs.keys():
          if not obj_attr['id'] in self.objectIds():
            print "[%s.initChildNodes]: Create %s"%(self.meta_type,obj_attr['id'])
            
            ##### Create ####
            obj = self.dGlobalAttrs[type]['obj_class']()
            obj.id = obj_attr['id']
            self._setObject(obj.id, obj)
            
            for oItem in self.objectValues():
              if oItem.id == obj.id:
                ##### Object State ####
                oItem.setObjStateNew(REQUEST)
                ##### Init Properties ####
                for s_lang in self.getLanguages():
                  oItem.setObjProperty('active',1,s_lang)
                ##### VersionManager ####
                oItem.__autoCommit__(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.zmsObjectValues
    # ---------------------------------------------------------------------------------------------
    def zmsObjectValues(self, meta_types=[]):
      rtn = []
      for child in self.objectValues():
        if child.meta_type[:3] == 'ZMS':
          rtn.append(child)
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.display_type
    # ---------------------------------------------------------------------------------------------
    def display_type(self, REQUEST=None, sMetatype=''):
      try:
        ob = self.getDocumentElement().getMetaobj(self.meta_id)
        rtn = ob['name']      
      except: 
        rtn = ZMSObject.display_type(self,REQUEST,sMetatype) # Inherit from Super-Class.
      return rtn


    """
    ###############################################################################################    
    #
    #   Eigenschaften
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSCustom.manage_changeProperties: 
    #
    #  Change Custom properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST, RESPONSE):
        """ ZMSCustom.manage_changeProperties """
        
        message = ''
        if REQUEST.has_key('btn') and REQUEST['btn'] != self.getLangStr('BTN_CANCEL',manage_lang):
        
          ##### Object State #####
          self.setObjStateModified(REQUEST)
            
          ##### Properties #####
          for key in self.getObjAttrs().keys():
            if not key in ['change_uid','change_dt']:
              self.setReqProperty(key,REQUEST)
          
          ##### VersionManager ####
          self.__autoCommit__(REQUEST)
          
          ##### Message #####
          message = self.getLangStr('MSG_CHANGED',manage_lang)
          
          # Return with message.
          self.checkIn(REQUEST)
          return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))
        
        # Return with message.
        self.checkIn(REQUEST)
        return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,urllib.quote(message),self.id))


    # -------------------------------------------------------------------------------------------------
    #	ZMSCustom.isPage:
    # -------------------------------------------------------------------------------------------------
    def isPage(self):
      return self.getType() == 'ZMSDocument'

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.isPageElement:
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self):
      return self.getType() == 'ZMSObject'

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getType
    # ---------------------------------------------------------------------------------------------
    def getType(self):
      o_metaobj = self.getMetaobj(self.meta_id)
      return o_metaobj['type']

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getDtmlTemplate
    # ---------------------------------------------------------------------------------------------
    def getDtmlTemplate(self):
      root = self.getDocumentElement().aq_parent
      meta_type = self.getType()
      if meta_type == 'ZMSObject':
        tmplt = '%s_%s'%(self.meta_type,self.meta_id)
      elif meta_type == 'ZMSDocument':
        tmplt = 'bodyContent%s_%s'%(self.meta_type,self.meta_id)
      elif meta_type == 'ZMSTeaserElement':
        tmplt = 'bodyContentTeaser%s_%s'%(self.meta_type,self.meta_id)
      dtml_tmplt = getattr(root,tmplt,None)
      return dtml_tmplt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.isResource
    # ---------------------------------------------------------------------------------------------
    def isResource(self,REQUEST): return self.getParentNode().isResource(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self,REQUEST): return self.getParentNode().getDCCoverage(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST):
      s_titleAlt = ""
      if self.isPage():
        try:
          attr = self.getDocumentElement().getMetaobj(self.meta_id)['attrs'][0]
          s_titleAlt = str(self.getObjProperty(attr['id'],REQUEST))
        except:
          pass
      if len(s_titleAlt) == 0: 
        s_titleAlt = self.display_type(REQUEST)
      return s_titleAlt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitle
    # ---------------------------------------------------------------------------------------------
    def getTitle(self,REQUEST): 
      s_title = ""
      if self.isPage():
        try:
          attr = self.getDocumentElement().getMetaobj(self.meta_id)['attrs'][1]
          s_title = str(self.getObjProperty(attr['id'],REQUEST))
        except:
          pass
      if len(s_title) == 0: 
        s_title = self.display_type(REQUEST)
      return s_title


    """
    ###############################################################################################    
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################    
    """

    # preload display interface for custom-objects
    render_default = HTMLFile('dtml/metaobj/render_default', globals()) 

    ###############################################################################################    
    #  ZMSCustom._getBodyContent:
    #         
    #  HTML presentation of Special Document. 
    ###############################################################################################    
    def _getBodyContent(self, REQUEST, thumbnail=0):
      html = ""
      try:
        dtml_tmplt = self.getDtmlTemplate()
        if self.getType() == 'ZMSObject':
          html = html + dtml_tmplt(self,REQUEST)
        elif self.getType() == 'ZMSDocument':
          html = html + dtml_tmplt(self,REQUEST)
        elif self.getType() == 'ZMSTeaserElement':
          html = html + '<table cellspacing="0" cellpadding="0" border="0" width="100%" class="teaserElement">\n'
          html = html + '<tr'
          if REQUEST.has_key('bgcolor_text'):
            html = html + ' class="%s"'%str(REQUEST['bgcolor_text'])
          html = html + '>\n'
          html = html + '<td align="left" valign="top" class="teaserText">'
          html = html + '<p class="teaserText">%s</p>'%dtml_tmplt(self,REQUEST)
          html = html + '</td>\n'
          html = html + '</tr>\n'
          html = html + '</table>\n'
      except:
        pass
      # Return <html>.
      return html

    ###############################################################################################    
    #  ZMSCustom.renderShort:
    #
    #  Renders short presentation of Special Document.
    ###############################################################################################    
    def renderShort(self, REQUEST):
      html = self._getBodyContent(REQUEST)
      if len(html) == 0 or self.getType() == 'ZMSDocument':
        html = self.render_default(self,ob=self,REQUEST=REQUEST)
      # Return <html>.
      return html

    ###############################################################################################    
    #	ZMSCustom.catalogText:
    #
    #	Catalog text of Special Document.
    ###############################################################################################    
    def catalogText(self, REQUEST):
      s = ""
      for attr in self.getMetaobj(self.meta_id)['attrs']:
        key = attr['id']
        type = attr['type']
        value = None
        if type in ['LangInt','LangString','LangText','LangHtml']:
          value = self.getObjProperty(key,REQUEST)
        elif type in self.dGlobalAttrs.keys():
          ob = getattr(self,key,None)
          if ob is not None:
            value = ob.catalogText(REQUEST)
        if value is not None and len(str(value)) > 0: s = s + str(value) + " "
      #++ print "[%s.catalogText]: %s"%(self.meta_type,s)
      return s

    ###############################################################################################    
    #	ZMSCustom.printHtml:
    #
    #	Renders print presentation of Special Document.
    ###############################################################################################    
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      html = self._getBodyContent(REQUEST)
      if len(html) == 0: 
        html = self.render_default(self,ob=self,REQUEST=REQUEST)
      # Return <html>.
      return html

###################################################################################################
